// ======================================================================
//
// Material.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef MATERIAL_H
#define MATERIAL_H

// ======================================================================

#include "CrcString.h"
#include "Material.def"

class Iff;
class MaterialData;
class Palette;

// ======================================================================

class StaticMaterial;
class Material;

class MaterialList
{
private:

	enum
	{
		MAT_MASK  = 0xff000000,
		MAT_RGB   = 0x7f000000
	};

private:

	// prevent any objects of this type from being created
	MaterialList(void);
	MaterialList(const MaterialList &);
	MaterialList &operator =(const MaterialList &);

private:

	static Material       *materialList;
	static int             materialCount;

	static real            time;

	static const Material       *debug;

	static const StaticMaterial *redMaterial;
	static const StaticMaterial *greenMaterial;
	static const StaticMaterial *blueMaterial;
	static const StaticMaterial *blackMaterial;
	static const StaticMaterial *whiteMaterial;

public:

	static void      install(void);
	static void      remove(void);

	static void      debugDump(void);

	static const Material       *debugMaterial(void);

	static const StaticMaterial *red(void);
	static const StaticMaterial *green(void);
	static const StaticMaterial *blue(void);
	static const StaticMaterial *black(void);
	static const StaticMaterial *white(void);

	static const Material       *fetch(const char *name);
	static const Material       *fetch(int number);
	static const StaticMaterial *fetch(int r, int g, int b);
	static const Material       *fetch(const Material *mat);
	static const Material       *fetch(Iff *iff);
	static void                  release(const Material *mat);

	static void      destroyAllData(void);
	static void      createAllData(void);

	static void      updateTime(real elapsed);
	static real      getTime(void);
};

class Material
{
	friend class MaterialList;
	friend class AnimatingMaterial;

protected:

	static int globalToken;

protected:

	CrcString   *crcName;
	char        *name;
	mutable int  usage;
	MaterialId   id;
	int          token;
	Material    *next;

private:

	// disable default constructor and assignment operator
	Material(void);
	Material &operator =(const Material &);

protected:

	Material(const char *newName, MaterialId newId);
	Material(const Material &material);
	virtual ~Material(void) = 0;

	virtual Material *copy(void) const;

public:

	const CrcString *getCrcName(void) const;
	const char *getName(void) const;
	MaterialId  getId(void) const;
	int         getToken(void) const;

	virtual const StaticMaterial *getStaticMaterial(void) const = 0;
};

class StaticMaterial : public Material
{
	friend class MaterialList;

public:

	enum Chromakey
	{
		ChromakeyOff,
		ChromakeyOn
	};

	enum Alpha
	{
		AlphaOff,
		AlphaConstant,
		AlphaMap
	};

	enum Color
	{
		ColorConstant,
		ColorMap
	};

private:

	// disable the copy constructor and the assignment operator
	StaticMaterial(void);
	StaticMaterial(const StaticMaterial &);
	StaticMaterial &operator =(const StaticMaterial &);

protected:

	MaterialData    *data;

	u_int8           r, g, b, a;

	int              width;
	int              height;
	int              mipMapLevels;

	const Palette   *palette;
	Pixel8         **rgbMap8;
	Pixel8         **alphaMap;

	Pixel32        **rgbMap32;

	Chromakey        chromakey;
	Alpha            alpha;
	Color            color;

protected:

	StaticMaterial(const char *newName, Iff *iff);
	virtual ~StaticMaterial(void);

	virtual Material *copy(void) const;

	void init(void);
	void load(Iff *iff);
	void load_0001(Iff *iff);
	void load_BITM(Iff *iff);
	void load_SALF(Iff *iff);

private:

	void createData(void);
	void destroyData(void);

public:

	Chromakey      getChromakey(void) const;
	Alpha          getAlpha(void) const;
	Color          getColor(void) const;

	// constant colors	
	u_int8         getR(void) const;
	u_int8         getG(void) const;
	u_int8         getB(void) const;
	u_int8         getA(void) const;

	// texture map dimensions
	int            getWidth(void) const;
	int            getHeight(void) const;
	int            getMipMapLevels(void) const;

	// texture map data
	const Palette *getPalette(void) const;
	const Pixel8  *getRGBMap8(int mipMapLevel=0) const;
	const Pixel8  *getAlphaMap(int mipMapLevel=0) const;
	const Pixel32 *getRGBMap32(int mipMapLevel=0) const;

	MaterialData  *getData(void) const;

	virtual const StaticMaterial *getStaticMaterial(void) const;
};

class AnimatingMaterial : public Material
{
	friend class MaterialList;

protected:

	real             born, rate;
	int              maxFrame;
	const Material **frame;

private:

	// disable the default constructor and the assignment operator
	AnimatingMaterial(void);
	AnimatingMaterial &operator =(const AnimatingMaterial &);

protected:
	
	AnimatingMaterial(const char *newName, Iff *iff);
	AnimatingMaterial(const AnimatingMaterial &);
	~AnimatingMaterial(void);

	virtual Material *copy(void) const;

	void load(Iff *iff);
	void load_0001(Iff *iff);
	void load_ANIM(Iff *iff);

public:

	int             getMaxFrame(void) const;
	int             getCurrentFrame(void) const;
	const Material *getFrame(int index) const;

	virtual const StaticMaterial *getStaticMaterial(void) const;
};

// ======================================================================

inline void MaterialList::updateTime(real elapsedTime)
{
	time += elapsedTime;
}

// ----------------------------------------------------------------------

inline real MaterialList::getTime(void)
{
	return time;
}

// ======================================================================

inline const char *Material::getName(void) const
{
	return name;
}

// ----------------------------------------------------------------------

inline const CrcString *Material::getCrcName(void) const
{
	return crcName;
}

// ----------------------------------------------------------------------

inline MaterialId Material::getId(void) const
{
	return id;
}

// ----------------------------------------------------------------------

inline int Material::getToken(void) const
{
	return token;
}

// ======================================================================

inline StaticMaterial::Chromakey StaticMaterial::getChromakey(void) const
{
	return chromakey;
}

// ----------------------------------------------------------------------

inline StaticMaterial::Alpha StaticMaterial::getAlpha(void) const
{
	return alpha;
}

// ----------------------------------------------------------------------

inline StaticMaterial::Color StaticMaterial::getColor(void) const
{
	return color;
}

// ----------------------------------------------------------------------

inline u_int8 StaticMaterial::getR(void) const
{
	return r;
}

// ----------------------------------------------------------------------

inline u_int8 StaticMaterial::getG(void) const
{
	return g;
}

// ----------------------------------------------------------------------

inline u_int8 StaticMaterial::getB(void) const
{
	return b;
}

// ----------------------------------------------------------------------

inline u_int8 StaticMaterial::getA(void) const
{
	return a;
}

// ----------------------------------------------------------------------

inline int StaticMaterial::getWidth(void) const
{
	return width;
}

// ----------------------------------------------------------------------

inline int StaticMaterial::getHeight(void) const
{
	return height;
}

// ----------------------------------------------------------------------

inline int StaticMaterial::getMipMapLevels(void) const
{
	return mipMapLevels;
}

// ----------------------------------------------------------------------

inline const Palette *StaticMaterial::getPalette(void) const
{
	return palette;
}

// ----------------------------------------------------------------------

inline const Pixel8 *StaticMaterial::getRGBMap8(int mipMapLevel) const
{
	return (rgbMap8 ? rgbMap8[mipMapLevel] : NULL);
}

// ----------------------------------------------------------------------

inline const Pixel8 *StaticMaterial::getAlphaMap(int mipMapLevel) const
{
	return (alphaMap ? alphaMap[mipMapLevel] : NULL);
}

// ----------------------------------------------------------------------

inline const Pixel32 *StaticMaterial::getRGBMap32(int mipMapLevel) const
{
	return (rgbMap32 ? rgbMap32[mipMapLevel] : NULL);
}

// ----------------------------------------------------------------------

inline MaterialData *StaticMaterial::getData(void) const
{
	return data;
}

// ======================================================================

inline int AnimatingMaterial::getMaxFrame(void) const
{
	return maxFrame;
}

// ----------------------------------------------------------------------

inline int AnimatingMaterial::getCurrentFrame(void) const
{
	return REAL_TO_INT((MaterialList::getTime() - born) * rate) % maxFrame;
}

// ----------------------------------------------------------------------

inline const Material *AnimatingMaterial::getFrame(int index) const
{
	return frame[index];
}

// ======================================================================

#endif
